@@ -53,6 +53,9 @@ public:
 
     friend class ProjectClip;
     friend class ThumbnailCache;
+    friend class KdenliveTests;
+    /** @brief The id of the bin where a drop operation happened */
+    QString dropBinSource;
     /** @brief Timer checking if we have missing clips in the project */
     QTimer missingClipTimer;
 
@@ -60,10 +63,12 @@ public:
     void buildPlaylist(const QUuid uuid);
 
     /** @brief Returns a clip from the hierarchy, given its id */
-    std::shared_ptr<ProjectClip> getClipByBinID(const QString &binId);
+    std::shared_ptr<ProjectClip> getClipByBinID(const QString &binId) const;
+    /** @brief Returns existing masks for a clip */
+    const QVector<MaskInfo> getClipMasks(const QString &binId) const;
     /** @brief Returns audio levels for a clip from its id */
-    const QVector <uint8_t>getAudioLevelsByBinID(const QString &binId, int stream);
-    double getAudioMaxLevel(const QString &binId, int stream);
+    const QVector<int16_t> getAudioLevelsByBinID(const QString &binId, int stream);
+    int16_t getAudioMaxLevel(const QString &binId, int stream);
 
     /** @brief Returns a list of clips using the given url */
     QStringList getClipByUrl(const QFileInfo &url) const;
@@ -74,7 +79,7 @@ public:
     /** @brief Gets a folder by its id. If none is found, nullptr is returned */
     std::shared_ptr<ProjectFolder> getFolderByBinId(const QString &binId);
     /** @brief Gets a list of all folders in this project */
-    QList <std::shared_ptr<ProjectFolder> > getFolders();
+    QList<std::shared_ptr<ProjectFolder>> getFolders();
     /** @brief Gets a id folder by its name. If none is found, empty string returned */
     const QString getFolderIdByName(const QString &folderName);
 
@@ -88,11 +93,11 @@ public:
     QStringList getEnclosingFolderInfo(const QModelIndex &index) const;
 
     /** @brief Deletes all element and start a fresh model */
-    void clean();
+    void clean(bool quit = false);
 
     /** @brief Returns the id of all the clips (excluding folders) */
     std::vector<QString> getAllClipIds() const;
-    
+
     /** @brief Updates the list of all created bin thumbnails */
     void updateCacheThumbnail(std::unordered_map<QString, std::vector<int>> &thumbData);
 
@@ -100,6 +105,7 @@ public:
     std::shared_ptr<ProjectFolder> getRootFolder() const;
 
     void loadSubClips(const QString &id, const QString &dataMap, Fun &undo, Fun &redo);
+    void loadSubSequences(const QString &id, const sequenceMap &dataMap, Fun &undo, Fun &redo);
 
     /** @brief Convenience method to retrieve a pointer to an element given its index */
     std::shared_ptr<AbstractProjectItem> getBinItemByIndex(const QModelIndex &index) const;
@@ -110,8 +116,8 @@ public:
     /** @brief Parse a bin playlist from the document tractor and reconstruct the tree
      *  @return A list of invalid sequence clips found in Project Bin (can be caused by 23.04.0 bug)
      */
-    QList<QUuid> loadBinPlaylist(Mlt::Service *documentTractor, std::unordered_map<QString, QString> &binIdCorresp, QStringList &expandedFolders,
-                                 QStringList &extraBins, const QUuid &activeUuid, int &zoomLevel);
+    QMap<QUuid, QString> loadBinPlaylist(Mlt::Service *documentTractor, std::unordered_map<QString, QString> &binIdCorresp, QStringList &expandedFolders,
+                                         QStringList &extraBins, const QUuid &activeUuid, int &zoomLevel);
     void loadTractorPlaylist(Mlt::Tractor documentTractor, std::unordered_map<QString, QString> &binIdCorresp);
 
     /** @brief Save document properties in MLT's bin playlist */
@@ -160,9 +166,12 @@ public:
        @param undo,redo: lambdas that are updated to accumulate operation.
        @param readyCallBack: lambda that will be executed when the clip becomes ready. It is given the binId as parameter
     */
-    bool requestAddBinClip(QString &id, const QDomElement &description, const QString &parentId, Fun &undo, Fun &redo,
-                           const std::function<void(const QString &)> &readyCallBack = [](const QString &) {});
-    bool requestAddBinClip(QString &id, const QDomElement &description, const QString &parentId, const QString &undoText = QString(), const std::function<void(const QString &)> &readyCallBack = [](const QString &) {});
+    bool requestAddBinClip(
+        QString &id, const QDomElement &description, const QString &parentId, Fun &undo, Fun &redo,
+        const std::function<void(const QString &)> &readyCallBack = [](const QString &) {});
+    bool requestAddBinClip(
+        QString &id, const QDomElement &description, const QString &parentId, const QString &undoText = QString(),
+        const std::function<void(const QString &)> &readyCallBack = [](const QString &) {});
 
     /** @brief This is the addition function when we already have a producer for the clip*/
     bool requestAddBinClip(
@@ -177,6 +186,8 @@ public:
     */
     bool requestAddBinSubClip(QString &id, int in, int out, const QMap<QString, QString> &zoneProperties, const QString &parentId, Fun &undo, Fun &redo);
     bool requestAddBinSubClip(QString &id, int in, int out, const QMap<QString, QString> &zoneProperties, const QString &parentId);
+    bool requestAddBinSubPlaylistClip(QString &id, int duration, const QMap<QString, QString> &zoneProperties, const QString &parentId, Fun &undo, Fun &redo);
+    bool requestAddBinSubPlaylistClip(QString &id, int duration, const QMap<QString, QString> &zoneProperties, const QString &parentId);
 
     /** @brief Request that a folder's name is changed
        @param clip : pointer to the folder to rename
@@ -189,6 +200,8 @@ public:
 
     /** @brief Request that the unused clips are deleted */
     bool requestCleanupUnused();
+    /** @brief Return a list of clips unused id */
+    const QStringList getUnusedClipIds() const;
 
     /** @brief Request that all clips using one of the given urls are removed from the project and deleted from the hard disk*/
     bool requestTrashClips(QStringList &ids, QStringList &urls);
@@ -238,7 +251,10 @@ public:
     QMap<QUuid, QString> getAllSequenceClips() const;
     /** @brief Return the main project tractor (container of all playlists) */
     std::shared_ptr<Mlt::Tractor> projectTractor();
-    const QString sceneList(const QString &root, const QString &filterData, Mlt::Tractor *activeTractor, int duration, const QString &aspectRatio = QString());
+    /** @brief Return the main sequence's xml as a playlist. If the render ratio is used and we use a tmp file containing the real xml data, return the tmp
+     * file's path as second parameter */
+    const std::pair<QString, QString> sceneList(const QString &root, const QString &filterData, Mlt::Tractor *activeTractor, int duration,
+                                                bool timelineProducerOnly = false, const QString &aspectRatio = QString());
     /** @brief Ensure that sequence @destUuid is not embedded in any dependency of sequence @srcUuid */
     bool canBeEmbeded(const QUuid destUuid, const QUuid srcUuid);
     /** @brief Store a newly created sequence tractor for reuse */
@@ -256,6 +272,9 @@ public:
     /** @brief Check that all sequences are correctly stored in the model */
     void checkSequenceIntegrity(const QString activeSequenceId);
     std::shared_ptr<EffectStackModel> getClipEffectStack(int itemId);
+    /** @brief Get the unique and unmutable uuid for this project clip */
+    const QString getBinClipUuid(const QString &binId) const;
+    const QString getBinClipIdByUuid(const QString uuid);
 
 protected:
     bool closing;
@@ -284,6 +303,8 @@ public Q_SLOTS:
     @param id is the id of the parent clip
     @param data is a definition of the subclips (keys are subclips' names, value are "in:out")*/
     void loadSubClips(const QString &id, const QString &clipData, bool logUndo);
+    void loadSubSequences(const QString &id, const sequenceMap &dataMap);
+    bool requestBinClipDeletionById(const QString &id);
 
 private Q_SLOTS:
     /** @brief Check how many invalid clips we have. */
@@ -325,10 +346,13 @@ Q_SIGNALS:
     void requestAudioThumbs(const QString &id, long duration);
     // TODO
     void markersNeedUpdate(const QString &id, const QList<int> &);
-    void itemDropped(const QStringList, const QModelIndex);
+    void itemDropped(const QStringList, const QModelIndex, bool dropFromSameSource);
     void urlsDropped(const QList<QUrl>, const QModelIndex);
     void effectDropped(const QStringList &, const QModelIndex &);
     void addTag(const QString &, const QModelIndex &);
     void addClipCut(const QString &, int, int);
     void resetPlayOrLoopZone(const QString &id);
+    void clipRenamed(const QString &controlUuid, const QString &clipName);
+    /** @brief emitted when a project clip is added / removed */
+    void projectClipsModified();
 };
